--
-- This file is a partially universal transformation logic for all of the characters.
-- 
--

---------------------------------------------------------------------
---------------- Functions and hook definitions  --------------------
---------------------------------------------------------------------

--- Jump starts the transformation sequence into their ALT_MODE for any transformer
--- @param player player_t
local function TransformIntoAltmode(player)

    if player.powers[pw_carry] ~= 0 then return end

     -- Apply basic alt properties that are performed by all transformers during transformation
     player.tf_form = TF_FORM.TF_TO_ALT_MODE -- Registering the transformation

     -- Apply the correct transformation process depending on specific transformer (skin)
     if(Valid(player.mo, SKINS.JAZZ)) then 
        player.mo.state = S_TF_JA_PON 
     elseif(Valid(player.mo, SKINS.STARSCREAM)) then
        player.mo.state = S_TF_ST_F2
     end

    player.in_tf_tics = states[player.mo.state].tics -- resetting transformation tics
end

--- Jump starts the transformation sequence into their BOT_MODE for any transformer
--- @param player player_t
local function TransformIntoBotmode(player)
    
    -- Apply basic bot properties
    player.height =      skins[player.mo.skin].height -- Fixing the shild height to match
    player.shieldscale = skins[player.mo.skin].shieldscale -- Fixing the shild scailing to match
    player.tf_form = TF_FORM.TF_TO_BOT_MODE -- Registering the transformation
    -- player.in_tf_tics = TRANFORMATION_TICS_MAX -- resetting transformation tics
    
    -- Apply the correct state 
    if(Valid(player.mo, SKINS.JAZZ)) then
        player.mo.state = S_TF_PON_JA
    elseif(Valid(player.mo, SKINS.STARSCREAM)) then
        player.mo.state = S_TF_F2_ST
    end

    player.in_tf_tics = states[player.mo.state].tics -- resetting transformation tics

end

---Evaluates whether the transformation into either robot or alt modes 
---will be completed in this tic
---@param player player_t
---@return boolean completed_transformation
local function TryCompleteTransformation(player)
    local completed_transformation = false
    -- If the transformation delay timer is completed
    if(player.in_tf_tics <= 0) then
        -- Transform into the alternative mode
        if(player.tf_form == TF_FORM.TF_TO_ALT_MODE) then
            player.tf_form = TF_FORM.ALT_MODE
            completed_transformation = true
        -- Transform into the standard robot mode
        elseif(player.tf_form == TF_FORM.TF_TO_BOT_MODE) then
            player.tf_form = TF_FORM.BOT_MODE
            completed_transformation = true
        end
    end
    return completed_transformation
end

---Immediately initiated the transformation sequence into the relevant mode 
---(depending on their current mode)
---@param player player_t
---@return boolean has_initiated_transformation
local function InitialteTransformation(player)
    
    local has_initiated_tf = false
    -- Transformation from the alt mode to the bot mode
    if(player.tf_form == TF_FORM.ALT_MODE) then
        TransformIntoBotmode(player)
        has_initiated_tf = true
    -- Transformation from the bot mode to the alt mode
    elseif(player.tf_form == TF_FORM.BOT_MODE) then
        TransformIntoAltmode(player)
        has_initiated_tf = true
    end 
    return has_initiated_tf
end

local function TransformationPlayerThink(player)

    --- Counts down how long player is in the transformation state
    if((player.tf_form == TF_FORM.TF_TO_ALT_MODE or player.tf_form == TF_FORM.TF_TO_BOT_MODE)
    and player.in_tf_tics > 0) then
        player.in_tf_tics = $-1
    end

    if(Active(player)) then
        
        -- Has the transformation been completed this frame?
        local transformed = TryCompleteTransformation(player)

        -- Retrieve the transformation input 
        local transform_input = (
                (player == secondarydisplayplayer and (input.gameControl2Down(GC_CUSTOM1) == 1 or input.gameControl2Down(GC_CUSTOM2) == 1))
                or (player ~= secondarydisplayplayer and (input.gameControlDown(GC_CUSTOM1) == 1 or input.gameControlDown(GC_CUSTOM2) == 1))
            )   
            and (player.tf_form == TF_FORM.ALT_MODE or player.tf_form == TF_FORM.BOT_MODE)

        ---Split screen decisions
        if(player == secondarydisplayplayer) then
            transform_input = transform_input or input.gameControl2Down(GC_CUSTOM1) == 1
        else
            transform_input = transform_input or input.gameControlDown(GC_CUSTOM1) == 1
            
        end
        -- Start to monitor for player inputs to transform if allowed
        if(player.climbing == 0 and player.mo.state ~= mobjinfo[player.mo.type].painstate and 
        transform_input == true and player.in_tf_tics <= 0) then
            InitialteTransformation(player)
        end

        
    end
end

local function TransformationPreThink(player)

    if(player.tf_form == TF_FORM.TF_TO_ALT_MODE or player.tf_form == TF_FORM.TF_TO_BOT_MODE) then
        player.pflags = $|PF_JUMPSTASIS
    end

    -- if(player.mo.state == S_TF_PON_JA) then
    --     -- player.mo.friction = FRACUNIT-FRACUNIT/50
    --     player.powers[pw_strong] = $|STR_GUARD
    -- end
    -- --While breakdancig, keep keep the momentum and guard up
    -- if(player.mo.state == S_TF_PON_JA_BR) then
    --     -- player.mo.friction = FRACUNIT-FRACUNIT/50
    --     player.powers[pw_strong] = $|STR_GUARD
    --     player.cmd.sidemove    = clamp($, -10, 10)
    --     player.cmd.forwardmove = clamp($, -10, 10)
    --     -- P_InstaThrust(player.mo, R_PointToAngle2(0, 0, player.mo.momx, player.mo.momy), 30*FRACUNIT)
    -- else
    --     player.powers[pw_strong] = $&~STR_GUARD
    -- end

end


local function TransformationPostThink(player)

    --Manually stopping the transforming animation
    if((player.mo.state == S_TF_JA_PON or player.mo.state == S_TF_PON_JA) 
        and player.mo.tics < states[player.mo.state].tics/2 and player.mo.frame & FF_FRAMEMASK == A) then
        player.mo.state = states[player.mo.state].nextstate
    end

    --Assign the correct state after transformation
    -- if(player.tf_form == TF_FORM.TF_TO_BOT_MODE) then
    --     if(not P_IsObjectOnGround(player.mo)) then
    --         states[player.mo.state].nextstate = S_PLAY_JUMP
    --     --Keep walking 
    --     elseif(player.mo.player.speed > FRACUNIT) then -- At low speed on the ground
    --         --Play walking animation
    --         print("run!")
    --         states[player.mo.state].nextstate = S_PLAY_WALK
    --     --Transforming at standstill
    --     else
    --         --Play normal animations
    --         states[player.mo.state].nextstate = S_PLAY_STND
    --     end
    -- end
end


local function TransformMoveCollide(jazzmo, collide_mo)
    
    --Hit the enemy while spinning and on guard
    if(L_ZCollide(jazzmo, collide_mo) and jazzmo.state == S_TF_PON_JA_BR and jazzmo.player.powers[pw_strong] & STR_GUARD) then
        P_DamageMobj(collide_mo, jazzmo, jazzmo, 1)
    --If transforming into the bot
    -- elseif (L_ZCollide(jazzmo, collide_mo) and jazzmo.state == S_TF_PON_JA) then
    --     P_DamageMobj(collide_mo, jazzmo, jazzmo, 1)
    end

end


---------------------------------------------------------------------
---------- Connecting all of the hooks from this file  --------------
---------------------------------------------------------------------


addHook("PlayerThink", function(player) 
    if(not TransformCharacter(player.mo)) then
        return 
    end
    TransformationPlayerThink(player) 
end)

addHook("PreThinkFrame", function() 
    for player in players.iterate() do
        if(not TransformCharacter(player.mo)) then
            return 
        end

        TransformationPreThink(player) 
    end
end)

addHook("PostThinkFrame", function() 
    for player in players.iterate() do
        if(not TransformCharacter(player.mo)) then
            return 
        end

        TransformationPostThink(player) 
    end
end)


addHook("MobjMoveCollide", function(mo, collide_mo)
    if(not TransformCharacter(mo)) then
        return --Transformer is not the one that is colliding
    end

    TransformMoveCollide(mo, collide_mo)

end, MT_PLAYER)

