
--------------- Hud function implementations ---------------

local function drawphasebars(v, stplayer, cam)
    
    if(stplayer.mo.skin ~= "starscream" or stplayer.tf_form ~= TF_FORM.ALT_MODE) then
		return false 
    end

    local emptybar = v.cachePatch("FUEL_BAR_EMPTY")
    local fullbar = v.cachePatch("FUEL_BAR_FULL")

    local xbarsize, ybarsize = 6*FRACUNIT, 16*FRACUNIT

    local x, y = 17*FRACUNIT, 55*FRACUNIT

    local xscale, yscale = 2*FRACUNIT, 2*FRACUNIT

		
	--Drawing sztatic empty bar 
	v.drawStretched(x, y, xscale, yscale, emptybar)
	--Drawing dynamic full bar 
	v.drawCropped(x, y, 
		xscale, yscale, 
		fullbar, 0, nil, 
		0, 0,
		xbarsize, FixedMul(FixedDiv(stplayer.mo.fuel_countdown, FUEL_CAP), ybarsize)
	)
	-- v.drawCropped(x, y, xscale, yscale, fullbar, 0, nil, 0, 0, xbarsize, ybarsize/4)
end

--------------- Hud function definitions ---------------

hud.add(drawphasebars, "game")


--Contains all functions of this file
--to be called outside of this file


local FORWARD_SPEED_MINIMUM = 1*FRACUNIT
local FORWARD_SPEED_CAP = 2*FORWARD_SPEED_MINIMUM
local FORWARD_ACCELERATION = 2*FRACUNIT
local FORWARD_NITRO = FORWARD_ACCELERATION*3
-- local FORWARD_ACCELERATION = 50*FRACUNIT

-- local SIDE_SPEED_CAP = FORWARD_SPEED_CAP/10
local SIDE_ACCELERATION = 3*FRACUNIT

local VERT_SPEED_CAP = FORWARD_SPEED_MINIMUM/2
local VERT_ACCELERATION = 2*FORWARD_ACCELERATION/2
local VERT_NITRO = VERT_ACCELERATION*3

local ROLL_STRENGTH = ANG1*10
				

---Executes the prethink for the Starscream In Alt mode
---@param player player_t
local function JetPreThink(player)

	-- player.mo.angle = $  -- retain current angle
	-- player.mo.rollangle = $  -- retain roll if you want

	-- These are custom variables on the mobj
	if(player.mo.rollturn == nil) then
		player.mo.rollturn = 0
	end
	if(player.mo.lastRollInput == nil) then
		player.mo.lastRollInput = 0
	end
	if(player.mo.lastRollTime == nil) then
		player.mo.lastRollTime = 0
	end


	--[[
	--Limiting side movement on input
	if(player.cmd.sidemove > SIDE_INPUT_CAP) then
		player.cmd.sidemove = SIDE_INPUT_CAP
	elseif(player.cmd.sidemove < -SIDE_INPUT_CAP) then
		player.cmd.sidemove = -SIDE_INPUT_CAP
	end
	]]--

end


-- --Rolling

-- --Rolling the sprite of the jet
-- -- player.mo.rollangle = clamp($ + (input_roll * ROLL_STRENGTH)/1000 - clamp($, -ROLL_STRENGTH/2, ROLL_STRENGTH/2), -ANGLE_90, ANGLE_90)
-- player.mo.rollangle = clamp($ + FixedAngle(input_roll * AngleFixed(ROLL_STRENGTH)/1000) - clamp($, -ROLL_STRENGTH/2, ROLL_STRENGTH/2), -ANGLE_90, ANGLE_90)

-- -- Move regularly if below limit
-- if(player.speed < FORWARD_SPEED_CAP) then
-- 	--Moving jet sideways, done through momentum assignment instead of relative thrusting to eliminate increased speed with forward movement
-- 	player.mo.momy = $ + FixedMul((input_roll*SIDE_ACCELERATION)/1000, sin(player.mo.angle + ANGLE_90))
-- 	player.mo.momx = $ + FixedMul((input_roll*SIDE_ACCELERATION)/1000, cos(player.mo.angle + ANGLE_90))
-- else
-- 	player.mo.momy = $- SIDE_ACCELERATION/10
-- 	player.mo.momx = $- SIDE_ACCELERATION/10
-- end

---Applies the jet momentum on the player with speified parameters
---@param player player_t
---@param pitch FRACUNIT
---@param roll FRACUNIT
local function ApplyThrottle(player, pitch, roll)
	local mo = player.mo

	-- === Initialize turn and pitch momentum ===
	if not mo.rollmom then mo.rollmom = 0 end
	if not mo.pitchmom then mo.pitchmom = 0 end

	-- Play engine sound
	if not S_SoundPlaying(mo, sfx_f22eng) then
		S_StartSound(mo, sfx_f22eng, player)
	end


	-- === Movement constants ===
	local thrust = 5 * FRACUNIT       
	local max_speed = 40 * FRACUNIT

	-- Rotation (roll/turn) constants
	local turn_accel = ANG1
	local turn_drag = ANG1/2
	local turn_speed_cap = ANG1*5

	-- Pitch movement constants
	local pitch_accel = FRACUNIT
	local pitch_drag = FRACUNIT / 2
	local pitch_cap = 5 * FRACUNIT

	-- === TURN MOMENTUM (roll) ===
	mo.rollmom = $ + FixedMul(roll, turn_accel)
	if roll == 0 then
		if mo.rollmom > 0 then
			mo.rollmom = $ - turn_drag
			if mo.rollmom < 0 then mo.rollmom = 0 end
		elseif mo.rollmom < 0 then
			mo.rollmom = $ + turn_drag
			if mo.rollmom > 0 then mo.rollmom = 0 end
		end
	end
	mo.rollmom = min(max(mo.rollmom, -turn_speed_cap), turn_speed_cap)

	-- === PITCH MOMENTUM (vertical) ===
	mo.pitchmom = $ + FixedMul(pitch, pitch_accel)
	if pitch == 0 then
		if mo.pitchmom > 0 then
			mo.pitchmom = $ - pitch_drag
			if mo.pitchmom < 0 then mo.pitchmom = 0 end
		elseif mo.pitchmom < 0 then
			mo.pitchmom = $ + pitch_drag
			if mo.pitchmom > 0 then mo.pitchmom = 0 end
		end
	end
	mo.pitchmom = min(max(mo.pitchmom, -pitch_cap), pitch_cap)

	-- === Apply turn momentum to angle ===
	mo.angle = $ + mo.rollmom
	player.drawangle = mo.angle


	-- === FORWARD MOVEMENT ===
	local forward_x = FixedMul(thrust, cos(mo.angle)) --+ FixedMul(extra*2, cos(mo.angle+ANGLE_90))
	local forward_y = FixedMul(thrust, sin(mo.angle)) --+ FixedMul(extra*2, sin(mo.angle+ANGLE_90))

	-- === Apply acceleration ===
	mo.momx = $ + forward_x
	mo.momy = $ + forward_y
	mo.momz = $ + mo.pitchmom


	-- === Clamp total vector magnitude ===
	local speed = FixedHypot(FixedHypot(mo.momx, mo.momy), mo.momz)
	if speed > max_speed then
		local scale = FixedDiv(max_speed, speed)
		mo.momx = FixedMul(mo.momx, scale)
		mo.momy = FixedMul(mo.momy, scale)
		mo.momz = FixedMul(mo.momz, scale)
	end

	-- === Visual roll effect (optional) ===
	mo.rollangle = mo.rollmom * 4
end




---Returns the normalized (-FRACUNIT .. FRACUNIT) Player's input controls, such as pitch, roll etc.
---@param player player_t
---@return integer input_roll, integer input_pitch
local function InputValue(player)

	-- Up and down movement (W and S keys)
	local input_pitch = 0
	-- Left and right movement (A and D keys)
	local input_roll = 0
	-- local input

	local input_pitch_keyboard = (input.gameControlDown(GC_BACKWARD) - input.gameControlDown(GC_FORWARD)) * 1000
	local input_pitch_joystick = clamp(input.joyAxis(JA_MOVE), -1000, 1000)

	local input_roll_keyboard = (input.gameControlDown(GC_STRAFELEFT) - input.gameControlDown(GC_STRAFERIGHT)) * 1000
	local input_roll_joystick = clamp(-input.joyAxis(JA_STRAFE), -1000, 1000)

	-- Choosing either the keyboard or joystick input for pitch
	if(abs(input_pitch_joystick) >= abs(input_pitch_keyboard)) then
		input_pitch = input_pitch_joystick
	else
		input_pitch = input_pitch_keyboard
	end
	input_pitch = FixedDiv(input_pitch, 1000)

	-- Choosing either the keyboard or joystick input for roll
	if(abs(input_roll_joystick) >= abs(input_roll_keyboard)) then
		input_roll = input_roll_joystick
	else
		input_roll = input_roll_keyboard
	end
	input_roll = FixedDiv(input_roll, 1000)
    
	return input_roll, input_pitch
end

---Executes the PlayerThink for the Starscream In Alt mode
---@param player player_t
local function JetPlayerThink(player)
	-- Player's input
	local input_roll, input_pitch = InputValue(player)
	ApplyThrottle(player, input_pitch, input_roll)
end

--Altering player inputs in jet mode
addHook("PreThinkFrame", function()
	for player in players.iterate() do
		if(Valid(player.mo, SKINS.STARSCREAM) and player.tf_form == TF_FORM.ALT_MODE) then
			player.pflags = $|PF_FULLSTASIS
			JetPreThink(player)
		end
	end
end)
		
addHook("PlayerThink",
	function(player)
		if(Valid(player.mo, SKINS.STARSCREAM) and player.tf_form == TF_FORM.ALT_MODE) then
			JetPlayerThink(player)
		end
	end)